/*************************************************************************\
*   				Xcard 1.2				  *
*									  *
* Copyright 1991, Kenneth C. Nelson			 	          *
*									  *
* Rights       : I, Ken Nelson own Xcard.  You may compile, modify, and   *
*                use Xcard.  You may not sell Xcard or anything derived   *
*		 from Xcard in binary or source for more than reasonable  *
*                media costs.                                             *
*                                                                         *
*		 Xcard uses a dynamic object library	   	          *
*		 that I don't own.  See the subdirectory dynobj for  	  *
*		 restrictions on its use.				  *
*								          *
*                Please feel free to modify Xcard. See Todo for details.  *
*									  *
\*************************************************************************/




/*
 * menu.c  - contains menu definitions, building and callback routines. !!
 *
 */



#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#ifdef XAW_NOT_NESTED
# include <X11/Box.h>
# include <X11/Paned.h>
# include <X11/SimpleMenu.h>
# include <X11/MenuButton.h>
# include <X11/Command.h>
# include <X11/Sme.h>
# include <X11/SmeBSB.h>
# include <X11/SmeLine.h>
#else
# include <X11/Xaw/Box.h>
# include <X11/Xaw/Paned.h>
# include <X11/Xaw/SimpleMenu.h>
# include <X11/Xaw/MenuButton.h>
# include <X11/Xaw/Command.h>
# include <X11/Xaw/Sme.h>
# include <X11/Xaw/SmeBSB.h>
# include <X11/Xaw/SmeLine.h>
#endif XAW not nested under /usr/include/X11

#include "cards.h"
#include "xwidgets.h"
#include "editbody.h"
#include "headers.h"
#include "search.h"
#include "xmsg.h"
#include "editor.h"


#define XCARD_BUTTON_WIDTH 60



/*
 * saveCurrCard() - saves the text,etc.. of the current card
 *		    before switching cards.
 *
 */

void saveCurrCard()
{
  int needs_update;

  needs_update = (strcmp(headerAreaText(),headerCurrentCard()) != 0);
  
  setHeaderCurrentCard(headerAreaText());
  setBodyCurrentCard(cardEditorText());

  if (needs_update)
  {
    updateCardHeaderList();
  }
  
}


/*
 * CBmenuAbout()  - the callback for the about button of the main menu.
 *
 */


#define MESSAGE_STRING \
"\n\n\n\
           XCard 1.2            \n\
               by               \n\
           Ken Nelson           \n\n\
  Send bugs to nelson@wrl.epi.com   \n\
\n\n\n\
"

/*ARGSUSED*/
void CBmenuAbout(w,client_data,call_data)
  Widget    w;
  XtPointer client_data;
  XtPointer call_data;
{
  popupDismissableMessage(w,MESSAGE_STRING);
}




/*
 * CBfirstCard() - the callback for the first card element of the card menu.
 *
 */

/*ARGSUSED*/
void CBfirstCard(w,client_data,call_data)
  Widget w;
  XtPointer client_data;
  XtPointer call_data;
{
  saveCurrCard();
  if (setCurrentCard(FIRSTCARD))
  {
    highlightHeader(currentCard());
  }
}




/*
 * CBlastCard() - the callback for the last card element of the card menu.
 *
 */

/*ARGSUSED*/
void CBlastCard(w,client_data,call_data)
  Widget w;
  XtPointer client_data;
  XtPointer call_data;
{
  saveCurrCard();
  if (setCurrentCard(LASTCARD))
  {
    highlightHeader(currentCard());
  }
}



static addCardDialogHandler(header,status)
  char *header;
  int  status;
{
  if (status)
  {
    saveCurrCard();
    addCard(APPEND,header,(char *)strdup(cardFileTemplate()));
    updateCardHeaderList();
    highlightHeader(currentCard());
  }
}

/*
 * CBaddCard() - the callback for the add card element of the card menu.
 *
 */

/*ARGSUSED*/
void CBaddCard(w,client_data,call_data)
  Widget w;
  XtPointer client_data;
  XtPointer call_data;
{
  char header[MAX_CARD_LINE+1];

  strcpy(header,"");
  popupDialog(w,"New Card Header: ",header,addCardDialogHandler);

}




/*
 * CBdeleteCard() - the callback for the delete card element of the card menu.
 *
 */

/*ARGSUSED*/
void CBdeleteCard(w,client_data,call_data)
  Widget w;
  XtPointer client_data;
  XtPointer call_data;
{
  popupDismissableMessage(w,"\n\n  Delete Card not implemented yet, sorry.  \n\n");
}




/*
 * CBnextButton() - the callback for the next button.
 *
 */

/*ARGSUSED*/
void CBnextButton(w,client_data,call_data)
  Widget w;
  XtPointer client_data;
  XtPointer call_data;
{
  saveCurrCard();
  if (setCurrentCard(NEXTCARD))
  {
    highlightHeader(currentCard());
  }
}


/*
 * CBprevButton() - the callback for the previous button.
 *
 */

/*ARGSUSED*/
void CBprevButton(w,client_data,call_data)
  Widget w;
  XtPointer client_data;
  XtPointer call_data;
{
  saveCurrCard();
  if (setCurrentCard(PREVCARD))
  {
    highlightHeader(currentCard());
  }
}



/* do nothing callback */

/*ARGSUSED*/
void CBmenuNop(w,client_data,call_data)
  Widget    w;
  XtPointer client_data;
  XtPointer call_data;
{
  popupDismissableMessage(w,"\n\n  Feature not implemented yet.  \n\n");
}



/* sort callback */

/*ARGSUSED*/
void CBmenuSort(w,order,call_data)
  Widget    w;
  int	    order;
  XtPointer call_data;
{
  sortCardFile(order);
  updateCardHeaderList();
}




/*
 * CBmenuSave()  - the callback for the save entry of the file menu.
 *
 */

/*ARGSUSED*/
void CBmenuSave(w,client_data,call_data)
  Widget    w;
  XtPointer client_data;
  XtPointer call_data;
{

  popupMessage(w,"\n\n     Saving...   \n\n");

  /*
   * First update the current selection
   *
   */

  saveCurrCard();

  /*
   * Then save the file to the existing cardfile.
   *
   */
   
  saveCardFile();

  popdownMessage();
}


/*
 * cardfileDialogHandler() - action passed to the popup dialog.
 *
 */
 
static void cardfileDialogHandler(filename,dialogStatus)
  char *filename;
  int  dialogStatus;
{
  if (dialogStatus)
  {
    if (!loadCardFile(filename,NEW_FILE))
    {
      popupDismissableMessage(toplevel,"\n\n  Could not open cardfile.  \n\n  Starting new one   \n\n");
    }
    updateCardHeaderList();
  }
}



/*
 * CBmenuLoad()  - the callback for the load entry of the File menu.
 *
 */

/*ARGSUSED*/
void CBmenuLoad(w,client_data,call_data)
  Widget    w;
  XtPointer client_data;
  XtPointer call_data;
{
  char       filename[256];
  
  strcpy(filename,cardFileName());

  popupDialog(w,"Load CardFile:",filename,cardfileDialogHandler);
    
}




/*
 * cardfileDialogHandler() - action passed to the popup dialog.
 *
 */
 
static void mergeCardfileDialogHandler(filename,dialogStatus)
  char *filename;
  int  dialogStatus;
{
  if (dialogStatus)
  {
    if (!loadCardFile(filename,MERGE_FILE))
    {
      popupDismissableMessage(toplevel,"\n\n  Could not open cardfile.  \n\n  Starting new one   \n\n");
    }
    updateCardHeaderList();
  }
}



/*
 * CBmergeLoad()  - the callback for the merge entry of the File menu.
 *
 */

/*ARGSUSED*/
void CBmenuMerge(w,client_data,call_data)
  Widget    w;
  XtPointer client_data;
  XtPointer call_data;
{
  char       filename[256];
  
  strcpy(filename,cardFileName());

  popupDialog(w,"Merge CardFile:",filename,mergeCardfileDialogHandler);
    
}




static void templateEditorAction(string,status)
  char *string;
  int  status;
{
  if (status)
  {
    setCardFileTemplate(string);
  }
}



/*
 * CBmenuTemplate()  - the callback for the edit template entry of the file menu.
 *
 */

/*ARGSUSED*/
void CBmenuTemplate(w,client_data,call_data)
  Widget    w;
  XtPointer client_data;
  XtPointer call_data;
{
  char text[MAX_CARD_SIZE+1];

  strcpy(text,cardFileTemplate());
  
  popupEditor(w,"Template Editor",text,templateEditorAction);
}





static void aboutEditorAction(string,status)
  char *string;
  int  status;
{
  if (status)
  {
    setCardFileAbout(string);
  }
}




/*
 * CBmenuAboutCardfile()  - the callback for the About entry of the file menu.
 *
 */

/*ARGSUSED*/
void CBmenuAboutCardfile(w,client_data,call_data)
  Widget    w;
  XtPointer client_data;
  XtPointer call_data;
{
  char text[MAX_CARD_SIZE+1];

  strcpy(text,aboutCardFile());
  
  popupEditor(w,"About Editor",text,aboutEditorAction);
}




/*
 * CBmenuQuit()  - the callback for the quit entry of the file menu.
 *
 */

/*ARGSUSED*/
void CBmenuQuit(w,client_data,call_data)
  Widget    w;
  XtPointer client_data;
  XtPointer call_data;
{

  exit(0);
  
}




/*
 * CBnewSearch()  - the callback for the new search entry of the search menu.
 *
 */

/*ARGSUSED*/
void CBnewSearch(w,client_data,call_data)
  Widget    w;
  XtPointer client_data;
  XtPointer call_data;
{

   popupSearchForm(w);
   
}





/*
 * addMenuArea() - builds and adds the menu area of Xcard. Attaches to the
 *		   parent widget that is passed in.
 *
 */

void  addMenuArea(parent)
  Widget parent;
{
   Widget menuArea;
   Widget button;
   Widget element;
   Widget menu;
   
   menuArea = XtVaCreateManagedWidget(
				     "menuArea",
				     boxWidgetClass,
				     parent,
				     XtNshowGrip,FALSE,
				     NULL
				    );
   XawPanedAllowResize(menuArea,FALSE);
   

   button = XtVaCreateManagedWidget(
				        "about",
					commandWidgetClass,
					menuArea,
					XtNlabel,"about",
					XtNshapeStyle,XmuShapeEllipse,
					NULL
				       );
  
   XtAddCallback(button,XtNcallback,CBmenuAbout,NULL);


   /*
    * Add the file menu button and its elements.
    *
    */

   button = XtVaCreateManagedWidget(
				    "file",
				    menuButtonWidgetClass,
				    menuArea,
				    XtNlabel,"file",
				    XtNshapeStyle,XmuShapeEllipse,
	 	   	  	    XtNwidth,XCARD_BUTTON_WIDTH,
				    NULL
				   );

   menu = XtCreatePopupShell("menu",
			     simpleMenuWidgetClass,
			     button,
			     NULL,
			     0
			    );


   element = XtVaCreateManagedWidget(
				     "save",
				     smeBSBObjectClass,
				     menu,
				     XtNlabel,"Save the CardFile",
				     NULL
				    );

   XtAddCallback(element,XtNcallback,CBmenuSave,NULL);


   element = XtVaCreateManagedWidget(
				     "",
				     smeLineObjectClass,
				     menu,
				     NULL
				    );

   element = XtVaCreateManagedWidget(
				     "load",
				     smeBSBObjectClass,
				     menu,
				     XtNlabel,"Load New CardFile",
				     NULL
				    );

   XtAddCallback(element,XtNcallback,CBmenuLoad,NULL);

   element = XtVaCreateManagedWidget(
				     "merge",
				     smeBSBObjectClass,
				     menu,
				     XtNlabel,"Merge in another CardFile",
				     NULL
				    );

   XtAddCallback(element,XtNcallback,CBmenuMerge,NULL);

   element = XtVaCreateManagedWidget(
				     "",
				     smeLineObjectClass,
				     menu,
				     NULL
				    );


   element = XtVaCreateManagedWidget(
				     "template",
				     smeBSBObjectClass,
				     menu,
				     XtNlabel,"Edit Template",
				     NULL
				    );

   XtAddCallback(element,XtNcallback,CBmenuTemplate,NULL);

   element = XtVaCreateManagedWidget(
				     "aboutCardfile",
				     smeBSBObjectClass,
				     menu,
				     XtNlabel,"About Cardfile",
				     NULL
				    );

   XtAddCallback(element,XtNcallback,CBmenuAboutCardfile,NULL);

   element = XtVaCreateManagedWidget(
				     "",
				     smeObjectClass,
				     menu,
				     NULL
				    );

   element = XtVaCreateManagedWidget(
				     "",
				     smeLineObjectClass,
				     menu,
				     NULL
				    );


   element = XtVaCreateManagedWidget(
				     "quit",
				     smeBSBObjectClass,
				     menu,
				     XtNlabel,"Quit XCard",
				     NULL
				    );

   XtAddCallback(element,XtNcallback,CBmenuQuit,NULL);



 
   /*
    * Add the search menu button and its elements.
    *
    */

   button = XtVaCreateManagedWidget(
				    "searchMenu",
				    menuButtonWidgetClass,
				    menuArea,
				    XtNlabel,"Search",
	 	   	  	    XtNwidth,XCARD_BUTTON_WIDTH,
				    XtNshapeStyle,XmuShapeEllipse,
				    NULL
				   );

   menu = XtCreatePopupShell("menu",
			     simpleMenuWidgetClass,
			     button,
			     NULL,
			     0
			    );

   element = XtVaCreateManagedWidget(
				     "newsearch",
				     smeBSBObjectClass,
				     menu,
				     XtNlabel,"Start New Search",
				     NULL
				    );

   XtAddCallback(element,XtNcallback,CBnewSearch,NULL);

   element = XtVaCreateManagedWidget(
				     "",
				     smeLineObjectClass,
				     menu,
				     NULL
				    );


   element = XtVaCreateManagedWidget(
				     "nextsearch",
				     smeBSBObjectClass,
				     menu,
				     XtNlabel,"Search For Next",
				     NULL
				    );

   XtAddCallback(element,XtNcallback,CBdoSearch,NULL);



   /*
    * Add the card menu button and its elements.
    *
    */

   button = XtVaCreateManagedWidget(
				    "cardMenu",
				    menuButtonWidgetClass,
				    menuArea,
				    XtNlabel,"Card",
	 	   	  	    XtNwidth,XCARD_BUTTON_WIDTH,
				    XtNshapeStyle,XmuShapeEllipse,
				    NULL
				   );

   menu = XtCreatePopupShell("menu",
			     simpleMenuWidgetClass,
			     button,
			     NULL,
			     0
			    );


   element = XtVaCreateManagedWidget(
				     "nextcard",
				     smeBSBObjectClass,
				     menu,
				     XtNlabel,"Next Card",
				     NULL
				    );

   XtAddCallback(element,XtNcallback,CBnextButton,NULL);

   element = XtVaCreateManagedWidget(
				     "prevcard",
				     smeBSBObjectClass,
				     menu,
				     XtNlabel,"Previous Card",
				     NULL
				    );

   XtAddCallback(element,XtNcallback,CBprevButton,NULL);

   element = XtVaCreateManagedWidget(
				     "",
				     smeLineObjectClass,
				     menu,
				     NULL
				    );

   element = XtVaCreateManagedWidget(
				     "firstcard",
				     smeBSBObjectClass,
				     menu,
				     XtNlabel,"First Card",
				     NULL
				    );

   XtAddCallback(element,XtNcallback,CBfirstCard,NULL);

   element = XtVaCreateManagedWidget(
				     "lastcard",
				     smeBSBObjectClass,
				     menu,
				     XtNlabel,"Last Card",
				     NULL
				    );

   XtAddCallback(element,XtNcallback,CBlastCard,NULL);

   element = XtVaCreateManagedWidget(
				     "",
				     smeLineObjectClass,
				     menu,
				     NULL
				    );

   element = XtVaCreateManagedWidget(
				     "addcard",
				     smeBSBObjectClass,
				     menu,
				     XtNlabel,"Add New Card",
				     NULL
				    );

   XtAddCallback(element,XtNcallback,CBaddCard,NULL);

   element = XtVaCreateManagedWidget(
				     "deletecard",
				     smeBSBObjectClass,
				     menu,
				     XtNlabel,"Delete Selected Card",
				     NULL
				    );

   XtAddCallback(element,XtNcallback,CBdeleteCard,NULL);

   element = XtVaCreateManagedWidget(
				     "",
				     smeLineObjectClass,
				     menu,
				     NULL
				    );

   element = XtVaCreateManagedWidget(
				     "saveAscii",
				     smeBSBObjectClass,
				     menu,
				     XtNlabel,"Save Card to ASCII file.",
				     NULL
				    );

   XtAddCallback(element,XtNcallback,CBmenuNop,NULL);

   element = XtVaCreateManagedWidget(
				     "savecard",
				     smeBSBObjectClass,
				     menu,
				     XtNlabel,"Load Card from ASCII file.",
				     NULL
				    );

   XtAddCallback(element,XtNcallback,CBmenuNop,NULL);





   /*
    * Add the sort menu button and its elements.
    *
    */

   button = XtVaCreateManagedWidget(
				    "sortMenu",
				    menuButtonWidgetClass,
				    menuArea,
				    XtNlabel,"Sort",
	 	   	  	    XtNwidth,XCARD_BUTTON_WIDTH,
				    XtNshapeStyle,XmuShapeEllipse,
				    NULL
				   );

   menu = XtCreatePopupShell("menu",
			     simpleMenuWidgetClass,
			     button,
			     NULL,
			     0
			    );


   element = XtVaCreateManagedWidget(
				     "ascendingOrder",
				     smeBSBObjectClass,
				     menu,
				     XtNlabel,"Ascending Order",
				     NULL
				    );

   XtAddCallback(element,XtNcallback,CBmenuSort,ASCENDING);

   element = XtVaCreateManagedWidget(
				     "descendingOrder",
				     smeBSBObjectClass,
				     menu,
				     XtNlabel,"Descending Order",
				     NULL
				    );

   XtAddCallback(element,XtNcallback,CBmenuSort,DESCENDING);


   /*
    * Now add buttons for prev and next cards. These functions are
    * also available from the menus, but these are common functions,
    * and it saves time for them to be here.
    */

   button = XtVaCreateManagedWidget(
				     "prevCard",
				     commandWidgetClass,
				     menuArea,
				     XtNshapeStyle,XmuShapeEllipse,
				     XtNlabel,"<-",
				     NULL
				    );
   XtAddCallback(button,XtNcallback,CBprevButton,NULL);


   button = XtVaCreateManagedWidget(
				     "nextCard",
				     commandWidgetClass,
				     menuArea,
				     XtNshapeStyle,XmuShapeEllipse,
				     XtNlabel,"->",
				     NULL
				    );
   XtAddCallback(button,XtNcallback,CBnextButton,NULL);

}
