/*************************************************************************\
*   				Xcard 1.2				  *
*									  *
* Copyright 1991, Kenneth C. Nelson			 	          *
*									  *
* Rights       : I, Ken Nelson own Xcard.  You may compile, modify, and   *
*                use Xcard.  You may not sell Xcard or anything derived   *
*		 from Xcard in binary or source for more than reasonable  *
*                media costs.                                             *
*                                                                         *
*		 Xcard uses a dynamic object library	   	          *
*		 that I don't own.  See the subdirectory dynobj for  	  *
*		 restrictions on its use.				  *
*								          *
*                Please feel free to modify Xcard. See Todo for details.  *
*									  *
\*************************************************************************/


/*
 * xmsg.c  - provides routines for popping up messages, with or without
 *	     a dismiss button.
 *
*/


#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>

#include <X11/Xaw/Label.h>
#include <X11/Xaw/Form.h>
#include <X11/Xaw/Command.h>
#include <X11/Xaw/AsciiText.h>

#include "xwidgets.h"

static Widget dismissableMessagePopup;	/* A popup with a dismiss button. */
static Widget dismissableMessageLabel;

static Widget popupDialogWidget;		/* A popup dialog with a text field. */
static Widget popupDialogLabel;
static Widget popupDialogText;



/*
 * A pointer to the status for the dialog popup.
 *
*/

int *dialogStatus;
void (*dialogAction)();

static Widget messagePopup;		/* A popup without a dismiss button. */
static Widget messageLabel;



/*ARGSUSED*/
static CBdismiss(w,client_data,call_data)
 Widget w;
 XtPointer client_data;
 XtPointer call_data;
{
  XtPopdown(dismissableMessagePopup);
}


/*ARGSUSED*/
static CBcancel(w,client_data,call_data)
 Widget w;
 XtPointer client_data;
 XtPointer call_data;
{
  char *string;

  XtVaGetValues(popupDialogText,"string",&string,NULL);
  XtPopdown(popupDialogWidget);
  if (dialogAction != NULL)
  {    
    (*dialogAction)(string,FALSE);
  }
}


/*ARGSUSED*/
static CBok(w,client_data,call_data)
 Widget w;
 XtPointer client_data;
 XtPointer call_data;
{
  char *string;
  
  XtVaGetValues(popupDialogText,"string",&string,NULL);
  XtPopdown(popupDialogWidget);
  if (dialogAction != NULL)
  {    
    (*dialogAction)(string,TRUE);
  }

}





/*
 * addMessagePopups - Adds the message popups.
 *
 */
 
void addMessagePopups(parent)
  Widget parent;
{

  Widget form,button,cancelButton,okButton;


 /*
  * An aciiTextWidget translation that disables carriage returns etc...
  *
  */

  static char oneLineTrans[] =  " Ctrl<Key>M : beginning-of-line()\n \
				 <Key>Return: beginning-of-line() ";


  XtTranslations  oneLineEditor;
  
  
  dismissableMessagePopup = XtCreatePopupShell(
					       "Popup",
					       transientShellWidgetClass,
					       toplevel,
					       NULL,
					       0
					      );
  form = XtVaCreateManagedWidget(
			       "dismissForm",
			       formWidgetClass,
			       dismissableMessagePopup,
			       NULL
			      );

  dismissableMessageLabel = XtVaCreateManagedWidget(
				  "dismissLabel",
				  labelWidgetClass,
				  form,
				  NULL
				 );

  button = XtVaCreateManagedWidget(
				   "dismissButton",
				   commandWidgetClass,
				   form,
				   XtNfromVert,dismissableMessageLabel,
				   XtNlabel,"Dismiss",
				   XtNshapeStyle,XmuShapeEllipse,
				   NULL
				  );

  XtAddCallback(button,XtNcallback,CBdismiss,NULL);




  /*
   * Now add the dialog Popup
   *
  */

 popupDialogWidget = XtCreatePopupShell(
				  "Popup",
				  transientShellWidgetClass,
				  toplevel,
				  NULL,
				  0
				 );
				 
  form = XtVaCreateManagedWidget(
			       "dismissForm",
			       formWidgetClass,
			       popupDialogWidget,
			       NULL
			      );

  popupDialogLabel = XtVaCreateManagedWidget(
				  "dialogLabel",
				  labelWidgetClass,
				  form,
				  XtNborderWidth,0,
				  NULL
				 );

  popupDialogText = XtVaCreateManagedWidget(
					    "dialogPopupText",
					    asciiTextWidgetClass,
					    form,
					    XtNwidth,400,
					    XtNeditType,XawtextEdit,
					    XtNfromHoriz,popupDialogLabel,
					    NULL
					   );

  /*
   * Set up prevention of carriage returns.
   *
  */
  
  oneLineEditor = XtParseTranslationTable(oneLineTrans);
  XtOverrideTranslations(popupDialogText,oneLineEditor);



  cancelButton = XtVaCreateManagedWidget(
				   "cancelDialogButton",
				   commandWidgetClass,
				   form,
				   XtNfromVert,dismissableMessageLabel,
				   XtNlabel,"Cancel",
				   XtNshapeStyle,XmuShapeEllipse,
				   NULL
				  );

  XtAddCallback(cancelButton,XtNcallback,CBcancel,NULL);

  okButton = XtVaCreateManagedWidget(
				   "okayDialogButton",
				   commandWidgetClass,
				   form,
				   XtNfromVert,dismissableMessageLabel,
				   XtNfromHoriz,cancelButton,
				   XtNlabel,"Ok",
				   XtNshapeStyle,XmuShapeEllipse,
				   NULL
				  );

  XtAddCallback(okButton,XtNcallback,CBok,NULL);





  /*
   * Now add the message popup that has no dismiss button
   *
   */
   
  messagePopup = XtCreatePopupShell(
				    "Popup",
				    transientShellWidgetClass,
				    toplevel,
				    NULL,
				    0
				   );

  messageLabel = XtVaCreateManagedWidget(
				  "Label",
				  labelWidgetClass,
				  messagePopup,
				  NULL
				 );

				   
					  
}


void popupMessage(where,message)
  Widget where;
  char   *message;
{

  Position x,y;

  if (where != NULL)
  {
    centerCoords(where,&x,&y);
    XtVaSetValues(messagePopup,
		  XtNx,x,
		  XtNy,y,
		  NULL
		 );
  }
  
    XtVaSetValues(messageLabel,
		  XtNlabel,message,
		  NULL
		 );
		 
    XtPopup(messagePopup,XtGrabExclusive);
  
  

}



void popdownMessage()
{
  XtPopdown(messagePopup);  
}




void popupDismissableMessage(where,message)
  Widget where;
  char   *message;
{
  Position x,y;

   if (where != NULL)
   {
     centerCoords(where,&x,&y);
     XtVaSetValues(dismissableMessagePopup,
	 	   XtNx,x,
		   XtNy,y,
		   NULL
		 );
   }

   XtVaSetValues(dismissableMessageLabel,
		  XtNlabel,message,
		  NULL
		 );

		 
    XtPopup(dismissableMessagePopup,XtGrabExclusive);

}




/*
 * popupDialog() - popup a dialog box, prompting message with
 *		   filling the useString. Status returns if
 *		   TRUE if OK pushed, FALSE if FALSE pushed.
 *		   If you don't care where to put dialog, set where to null.
 *		   useString should be big enought to hold whatever you
 *		   want, and may have an initial value.
 */
 
Widget popupDialog(where,message,useString,action)
  Widget   where;
  char    *message;
  char    *useString;
  void    (*action)();
{
  Position x,y;

  if (where != NULL)
  {
    centerCoords(where,&x,&y);
    XtVaSetValues(popupDialogWidget,
		  XtNx,x,
		  XtNy,y,
		  NULL
		 );		 
  }

  XtVaSetValues(
		popupDialogLabel,
		XtNlabel,message,
		NULL
	       );

  XtVaSetValues(
		popupDialogText,
		XtNstring,useString,
		NULL
	       );

  dialogAction = action;

  XtPopup(popupDialogWidget,XtGrabExclusive);

}
  
